﻿//C# Example
using UnityEditor;
using UnityEngine;

public class TilesetEditor : EditorWindow
{
	Tileset tileset;
	Texture lineTexture;
	int mode;
	string[] modes = {"height", "width", "angle"};
	bool viewProperties;
	int drawY;
	string filename;
	
	// Add menu item named "My Window" to the Window menu
	[MenuItem("Window/Tileset Editor")]
	public static void ShowWindow()
	{
		//Show existing window instance. If one doesn't exist, make one.
		EditorWindow.GetWindow(typeof(TilesetEditor));
	}

	void OnGUI()
	{
		drawY = 80;
		if(lineTexture == null){
			lineTexture = Resources.Load ("TilesetEditorLineTexture") as Texture;
		}

		filename = EditorGUILayout.TextField("filename", filename);


		if(GUILayout.Button("Save")){
			tileset.Save(filename);
		}
		if(GUILayout.Button("Load")){
			
			tileset = new Tileset();
			tileset.Load(filename);
			Repaint ();
			return;
		}
		if(GUILayout.Button("New")){
			New();
		}
		drawY += 45;

		
		if(tileset == null){
			tileset = new Tileset();
			Repaint ();
			return;
		}

		tileset.Reset();

		viewProperties = EditorGUILayout.ToggleLeft("Hide Details", viewProperties);
		if(!viewProperties){
			tileset.texture = EditorGUILayout.ObjectField("Texture", tileset.texture, typeof(Texture), false) as Texture;
			tileset.cols = EditorGUILayout.IntField("Columns", tileset.cols);
			tileset.rows = EditorGUILayout.IntField("Rows", tileset.rows);
			tileset.tileResolution = EditorGUILayout.IntField("Tile resolution", tileset.tileResolution);
			drawY += 160;
			EditorGUILayout.Separator();
		}
		mode = EditorGUILayout.Popup("Mode", mode, modes);
		if(GUI.changed){
			Repaint ();
			return;
		}

		if( tileset.texture != null){
			
			if(tileset.cols <= 0 || tileset.rows <= 0){
				EditorGUILayout.LabelField("Tileset rows and columns must be positive values!");
				Debug.LogError("Tileset rows and columns must be positive values.");
				return;
			}

			if(tileset.cols <= 0 || tileset.texture.width % tileset.cols != 0){
				EditorGUILayout.LabelField("Warning: texture width does not divide evenly into "
				                           + tileset.cols + " columns!");
				drawY += 20;
			}
			if(tileset.rows <= 0 || tileset.texture.height % tileset.rows != 0){
				EditorGUILayout.LabelField("Warning: texture height does not divide evenly into "
				                           + tileset.rows + " rows!");
				drawY += 20;
			}
			int drawWidth = (int)position.width - 10, drawHeight = (int)position.height - drawY - 5;
			EditorGUI.DrawPreviewTexture(new Rect(5, drawY, drawWidth, drawHeight), tileset.texture);

			int tileW = drawWidth / tileset.cols, tileH = drawHeight / tileset.rows;

			for(int i = 1; i < tileset.rows; i++){
				EditorGUI.DrawPreviewTexture(new Rect(5, drawY + i * tileH, drawWidth, 1), lineTexture);
			}
			for(int j = 1; j < tileset.cols; j++){
				EditorGUI.DrawPreviewTexture(new Rect(5 + j * tileW, drawY, 1, drawHeight), lineTexture);
			}
			if(mode == 0){
				float resWidth = (float)tileW / tileset.tileResolution;

				//mouse input:
				if(Event.current.type == EventType.MouseDrag || Event.current.type == EventType.MouseDown){
					Vector2 pos = Event.current.mousePosition;
					int i = (int)((pos.x - 5) / tileW);
					int j = (int)((pos.y - drawY) / tileH);
					if( pos.y > drawY && i >= 0 && i < tileset.cols && j >= 0 && j < tileset.rows){
						int k = (int)((pos.x - 5 - i * tileW) * tileset.tileResolution / tileW);
						short newValue = 0;
						if(Event.current.button == 0){
							newValue = (short)(255 - (pos.y - drawY - j * tileH) * 255f / tileH);
						}
						tileset.tiles[i,j].heights[k] = newValue;
						Repaint();
					}
				}


				//draw lines:
				for(int i = 0; i < tileset.cols; i++){
					for(int j = 0; j < tileset.rows; j++){
						for(int k = 0; k < tileset.tileResolution; k++){
							if(tileset.tiles == null){
								Debug.Log ("missing tile list" + i + "," + j);
								return;
							}
							if(tileset.tiles[i,j] == null){
									Debug.Log ("missing tile " + i + "," + j);
								return;
							}
							if(tileset.tiles[i,j].heights == null){
								Debug.Log ("missing tile heights" + i + "," + j);
								return;
							}

							EditorGUI.DrawPreviewTexture(new Rect(5 + i * tileW + k * resWidth,
							                                      drawY + j * tileH + (tileH - tileset.tiles[i,j].heights[k] / 255f * tileH),
							                                      resWidth, 1),
							                             lineTexture);
						}
					}
				}
			}else if(mode == 1){
				float resHeight = (float)tileH / tileset.tileResolution;
				
				//mouse input:
				if(Event.current.type == EventType.MouseDrag || Event.current.type == EventType.MouseDown){
					Vector2 pos = Event.current.mousePosition;
					int i = (int)((pos.x - 5) / tileW);
					int j = (int)((pos.y - drawY) / tileH);
					if( pos.y > drawY && i >= 0 && i < tileset.cols && j >= 0 && j < tileset.rows){
						int k = (int)((pos.y - drawY - j * tileH) * tileset.tileResolution / tileH);
						short newValue = 0;
						if(Event.current.button == 0){
							newValue = (short)((pos.x - 5 - i * tileW) * 255f / tileW);
						}
						tileset.tiles[i,j].widths[k] = newValue;
						Repaint();
					}
				}
				
				
				//draw lines:
				for(int i = 0; i < tileset.cols; i++){
					for(int j = 0; j < tileset.rows; j++){
						for(int k = 0; k < tileset.tileResolution; k++){
							EditorGUI.DrawPreviewTexture(new Rect(5 + i * tileW + tileset.tiles[i,j].widths[k] * tileW / 255f, 
							                                      drawY + j * tileH + k * resHeight, 1, resHeight),
							                             lineTexture);
						}
					}
				}

			}else{

			}


		}
		/*
		groupEnabled = EditorGUILayout.BeginToggleGroup ("Optional Settings", groupEnabled);
		myBool = EditorGUILayout.Toggle ("Toggle", myBool);
		myFloat = EditorGUILayout.Slider ("Slider", myFloat, -3, 3);
		EditorGUILayout.EndToggleGroup ();
		*/
	}
	void New(){
		tileset = new Tileset();
		tileset.Reset();
	}


}



