﻿using UnityEngine;
using System.Collections;

public class Runner : MonoBehaviour {
	public enum Dir {DOWN, RIGHT, UP, LEFT};
	public GameObject topLeft, topRight, left, right, bottomLeft, bottomRight;
	bool[] groundChecks = {false, false, false, false};
	bool grounded;
	Dir downDir = Dir.DOWN;
	float downAngle = 0;
	float groundSpeed;
	Vector2 speed;
	public float gravity;
	public float playerSpeed;
	public float playerMaxSpeed;
	public float playerAirSpeed;
	public float friction;

	public TiledLevel level;

	// Use this for initialization
	void Start () {
	
	}
	
	// Update is called once per frame
	void Update () {
		//input, gravity:
		//*
		float xInput = Input.GetAxis ("Horizontal");
		if(friction * Time.deltaTime > Mathf.Abs (speed.x)){
			speed.x = 0f;
		}else{
			speed.x -= Mathf.Sign(speed.x) * friction * Time.deltaTime;
		}
		if(grounded){
			speed.x += xInput * Mathf.Cos(downAngle) * playerSpeed * Time.deltaTime;
			speed.y += xInput * Mathf.Sin(downAngle) * playerSpeed * Time.deltaTime;
		}else{
			speed.x += xInput * playerAirSpeed * Time.deltaTime;
			speed.y -= gravity * Time.deltaTime;
		}
		if(speed.sqrMagnitude > playerMaxSpeed * playerMaxSpeed){
			speed.Normalize();
			speed *= playerMaxSpeed;
		}
		//*/

		//get tile for each sensor, find col/row height/width, displace Runner as neccesary. 
		grounded = false;
		groundChecks[0] = groundChecks[1] = groundChecks[2] = groundChecks[3] = false;

		SensorTest (topLeft.transform.position, Dir.UP);
		SensorTest (topRight.transform.position, Dir.UP);
		SensorTest (bottomLeft.transform.position, Dir.DOWN);
		SensorTest (bottomRight.transform.position, Dir.DOWN);
		SensorTest (left.transform.position, Dir.LEFT);
		SensorTest (right.transform.position, Dir.RIGHT);

		Vector3 sensorPos;
		sensorPos = bottomLeft.transform.position;

		//stop your speed if you hit a wall:
		if(groundChecks[(int)Dir.DOWN]){
			speed.y = Mathf.Max(speed.y, 0);
		}
		if(groundChecks[(int)Dir.RIGHT]){
			speed.x = Mathf.Max(speed.x, 0);
		}
		if(groundChecks[(int)Dir.UP]){
			speed.y = Mathf.Min(speed.y, 0);
		}
		if(groundChecks[(int)Dir.LEFT]){
			speed.x = Mathf.Min(speed.x, 0);
		}

		transform.Translate(speed * Time.deltaTime);
	}

	public void SensorTest(Vector3 sensorPos, Dir sensorDir){
		sensorPos -= level.transform.position;
		Dir dir = (Dir)(((int)sensorDir + (int)downDir) % 4);

		int resolution = level.tileset.tileResolution;
		Tile tile = level.GetTile(sensorPos);
		float ground;

		if( dir == Dir.DOWN || dir == Dir.UP){
			ground = tile.heights[(int)((sensorPos.x % level.tileSize.x) * resolution / level.tileSize.x)] * level.tileSize.y / 255;
			if(ground <= 0f){
				return;	//ignore this.
			}
			ground += sensorPos.y - sensorPos.y % level.tileSize.y;
		}else{
			ground = tile.widths [(int)((sensorPos.y % level.tileSize.y) * resolution / level.tileSize.y)] * level.tileSize.x / 255;
			if(ground <= 0f){
				return;	//ignore this.
			}
			ground += sensorPos.x - sensorPos.x % level.tileSize.x;
		}
		
		if(dir == Dir.DOWN && ground >= sensorPos.y){
			transform.Translate(Vector3.up * (ground - sensorPos.y));
			grounded = groundChecks[(int)Dir.DOWN] = true;

		}else if(dir == Dir.UP && ground <= sensorPos.y){
			transform.Translate(Vector3.down * (sensorPos.y - ground));
			grounded = groundChecks[(int)Dir.UP] = true;

		}else if(dir == Dir.RIGHT && ground <= sensorPos.x){
			transform.Translate(Vector3.left * (sensorPos.x - ground));
			grounded = groundChecks[(int)Dir.RIGHT] = true;

		}else if(dir == Dir.LEFT && ground >= sensorPos.x){
			transform.Translate(Vector3.right * (ground - sensorPos.x));
			grounded = groundChecks[(int)Dir.LEFT] = true;

		}
	}
}
