﻿using UnityEngine;
using System.IO;
using System.Collections;

public class Tileset{
	public Texture texture;
	public int cols=1, rows=1;
	public int tileResolution=4;
	public Tile[,] tiles;

	public Tileset(){
		Reset ();
	}

	public void Reset(){
		if(cols <= 0 || rows <= 0){
			return;
		}
		if(tiles == null){
			tiles = new Tile[cols, rows];
			for(int i = 0; i < cols; i++){
				for(int j = 0; j < rows; j++){
					tiles[i,j] = new Tile(tileResolution);
				}
			}
		}else if(tiles.GetLength(0) != cols || tiles.GetLength(1) != rows 
		         || tiles[0,0].heights.Length != tileResolution){
			Resize();
		}
	}

	public void Resize(){
		Tile[,] oldTiles = tiles;
		tiles = new Tile[cols, rows];
		for(int i = 0; i < cols; i++){
			for(int j = 0; j < rows; j++){
				if( i < oldTiles.GetLength(0) && j < oldTiles.GetLength(1) 
				   && oldTiles[i, j].heights.Length == tileResolution){
					tiles[i,j] = oldTiles[i,j];
				}else{
					tiles[i,j] = new Tile(tileResolution);
				}
			}
		}
	}

	
	public void Load(string name){
		
		StreamReader file = new StreamReader("Assets/Tilesets/" + name + ".tiles");
		texture = Resources.Load(file.ReadLine().Trim()) as Texture;
		cols = int.Parse(file.ReadLine());
		rows = int.Parse (file.ReadLine());
		tileResolution = int.Parse (file.ReadLine());
		int tileW = texture.width / cols, tileH = texture.height / rows;
		
		tiles = new Tile[cols, rows];
		int idx = 1;
		for(int i = 0; i < cols; i++){
			for(int j = 0; j < rows; j++){
				tiles[i,j] = new Tile(tileResolution);
				tiles[i,j].angle = short.Parse(file.ReadLine());
				for(int k = 0; k < tileResolution; k++){
					tiles[i,j].heights[k] = short.Parse(file.ReadLine ());
				}
				for(int k = 0; k < tileResolution; k++){
					tiles[i,j].widths[k] = short.Parse(file.ReadLine());
				}
				tiles[i,j].sprite = Sprite.Create((Texture2D)texture, 
				                                  new Rect(tileW * i, tileH *(rows -  j - 1), tileW, tileH), 
				                                  new Vector2(0f, 0f));
			}
		}
	}
	
	public void Save(string name){
		string text = texture.name + "\n";
		text += cols + "\n";
		text += rows + "\n";
		text += tileResolution + "\n";
		foreach(Tile tile in tiles){
			text += tile.angle + "\n";
			foreach(short height in tile.heights){
				text += height + "\n";
			}
			foreach(short width in tile.widths){
				text += width + "\n";
			}
		}
		File.WriteAllText("Assets/Tilesets/" + name + ".tiles", text);
	}
}
